/**********
 * Copyright (c) 2003-2004 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

#include "armcalls.h"
#include "bn/bn_armcalls.h"
#include "aes/aes_armcalls.h"


#include <Standalone.h>

STANDALONE_CODE_RESOURCE_ID(1000);


uint32_t start/*dispatch_from_68K*/(const void *emulStateP, 
                                    void *userData68KP, 
                                    Call68KFuncType *call68KFuncP)
{
    int i;
    uint32_t result = 0;
    int done = 0;
    pace_args *p68K = (pace_args *)userData68KP;

    for (i = 0; i < MAX_PARAMS; i++) {
        p68K->param[i] = ByteSwap32(p68K->param[i]);
    }
    p68K->globals = ByteSwap32(p68K->globals);
    p68K->fn = ByteSwap32(p68K->fn);

    p68K->globals->emulState = emulStateP;
    p68K->globals->call68K = call68KFuncP;
    for (i = 0; i < MAX_GLOBALS; i++) {
        p68K->globals->global[i] = ByteSwap32(p68K->globals->global[i]);
    }

    if (p68K->fn > BN_FIRST_ID  &&  p68K->fn < BN_LAST_ID) {
        result = BN_dispatch_from_68K(p68K->fn, p68K->globals, &p68K->param[0]);
    } else if (p68K->fn > AES_FIRST_ID  &&  p68K->fn < AES_LAST_ID) {
        result = AES_dispatch_from_68K(p68K->fn, p68K->globals, &p68K->param[0]);
    } else {
        // fixme die here
        result = 0;
    }

    // swap globals back to 68K-endian
    for (i = 0; i < MAX_GLOBALS; i++) {
        p68K->globals->global[i] = ByteSwap32(p68K->globals->global[i]);
    }

    return result;
}
