/**********
 * Copyright (c) 2003-2004 Greg Parker.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY GREG PARKER ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 **********/

// recordlist.h
// A database, a table, and a scroll bar.


#ifndef RECORDLIST_H
#define RECORDLIST_H

#include "includes.h"

#define noRecord ((UInt16)0xffff)

typedef struct RecordList RecordList;

typedef void (*RecordListDrawProc)(MemPtr record, UInt16 index, 
                                   RectanglePtr bounds);

RecordList *RecordListNew(DmOpenRef newDB, UInt16 newFormID, 
                          UInt16 newTableID, UInt16 newScrollbarID, 
                          RecordListDrawProc newDraw);
void RecordListFree(RecordList *rl);
void RecordListUpdate(RecordList *rl);
UInt16 RecordListCount(RecordList *rl);

UInt16 RecordListSelectedIndex(RecordList *rl);
void RecordListSetSelectedIndex(RecordList *rl, UInt16 index);
void RecordListClearSelection(RecordList *rl);

MemHandle RecordListQuerySelectedRecord(RecordList *rl);
MemHandle RecordListGetSelectedRecord(RecordList *rl, UInt32 createWithSize);
void RecordListReleaseRecord(RecordList *rl, MemHandle recordH, Boolean dirty);
void RecordListDeleteSelectedRecord(RecordList *rl);

MemHandle RecordListQueryIndexedRecord(RecordList *rl, UInt16 index);
MemHandle RecordListGetIndexedRecord(RecordList *rl, UInt16 index);
void RecordListDeleteIndexedRecord(RecordList *rl, UInt16 index);

Boolean RecordListHandleEvent(RecordList *rl, EventPtr event);

#endif
